--
-- OrganizedPanic
-- 
-- Copyright (c) 2021 Dana Basken
--

local VERSION="9.0.5.4"

local _, NS = ...

OrganizedPanic = LibStub("AceAddon-3.0"):NewAddon(NS.ADDON_NAME, "AceConsole-3.0", "AceTimer-3.0", "AceEvent-3.0", "AceComm-3.0")

print(PrefixString(NS.ADDON_NAME, ("v%s"):format(VERSION), NS.PREFIX_COLOR))

local OPTIONS = NS:BuildOptions(NS.ADDON_NAME, OrganizedPanic)

local AceGUI = LibStub("AceGUI-3.0") -- needed for AceConfigDialog
local AceEvent = LibStub("AceEvent-3.0")
local AceTimer = LibStub("AceTimer-3.0")

function OrganizedPanic:OnInitialize()
  self.db = LibStub("AceDB-3.0"):New("OrganizedPanicDB", NS.DEFAULTS, true)

  LibStub("AceConfig-3.0"):RegisterOptionsTable(NS.ADDON_NAME, OPTIONS)
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions(NS.ADDON_NAME)
  
  self:CreateFlasher()

  self.castingStates = {}
  self.name = UnitName("player")

  OrganizedPanic:RegisterEvent("UNIT_SPELLCAST_START")
  OrganizedPanic:RegisterEvent("UNIT_SPELLCAST_STOP")
  OrganizedPanic:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
  OrganizedPanic:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")

  OrganizedPanic:RegisterChatCommand(NS.CHAT_COMMAND_NAME, "HandleChatCommand")
end

function OrganizedPanic:OnEnable()
end

function OrganizedPanic:UNIT_SPELLCAST_START(event, unit, guid, id)
  if self:SpellCanPanic(unit, id) then
    self:SetCastingState(UnitName(unit), event, GetSpellInfo(id))
  end
end

function OrganizedPanic:UNIT_SPELLCAST_STOP(event, unit, guid, id)
  if self:SpellCanPanic(unit, id) then
    self:SetCastingState(UnitName(unit), event, GetSpellInfo(id))
  end
end

function OrganizedPanic:UNIT_SPELLCAST_CHANNEL_START(event, unit, guid, id)
  if self:SpellCanPanic(unit, id) then
    self:SetCastingState(UnitName(unit), event, GetSpellInfo(id))
  end
end

function OrganizedPanic:UNIT_SPELLCAST_CHANNEL_STOP(event, unit, guid, id)
  if self:SpellCanPanic(unit, id) then
    self:SetCastingState(UnitName(unit), event, GetSpellInfo(id))
  end
end

function OrganizedPanic:SpellCanPanic(unit, id)
  -- does the spell have a "panic" entry?
  local role = UnitGroupRolesAssigned(unit)
  if not NS.PANIC_SPELLS[spell] then return end
  -- if debugging, act like we're in raid and bypass "self" check
  if self.db.profile.debug then return true end
  -- check for raid group
  if not self:InRaid() then return end
  -- check to see if caster is me
  local caster = UnitName(unit)
  return caster ~= self.name
end

function OrganizedPanic:InRaid()
  local isInInstance, instanceType = IsInInstance();
  return instanceType == "raid"
end

function OrganizedPanic:SetCastingState(caster, event, spell)
  if not self.castingStates[caster] then
    self.castingStates[caster] = {event = nil, spell = nil}
  end
  if self.castingStates[caster].spell ~= spell or self.castingStates[caster].event ~= event then
    self.castingStates[caster] = {event = event, spell = spell}
    self:OnCastingStateChanged(caster)
  end
end

function OrganizedPanic:OnCastingStateChanged(caster)
  local state = self.castingStates[caster]
  if self.db.profile.debug then
    print(("STATE CHANGE: %s %s %s"):format(caster, state.event, state.spell))
  end
  if state.event == "UNIT_SPELLCAST_START" or state.event == "UNIT_SPELLCAST_CHANNEL_START" then
    self:PlayAlertSound()
    self:ShowRaidWarning(caster, state.spell)
    self:FlashBorder()
  end
end

function OrganizedPanic:CreateFlasher()
  self.frame = CreateFrame("Frame", nil, UIParent, BackdropTemplateMixin and "BackdropTemplate")
  self.frame:SetFrameStrata("BACKGROUND");
  self.frame:SetClampedToScreen(true)
  self.frame:SetAllPoints(UIParent)

  local texture = self.frame:CreateTexture(nil,"BACKGROUND");
  texture:SetTexture("Interface\\NSs\\OrganizedPanic\\images\\border.tga");
  texture:SetBlendMode("ADD");
  texture:SetVertexColor(self.db.profile.borderColor.r, self.db.profile.borderColor.g, self.db.profile.borderColor.b);
  texture:SetAllPoints(self.frame);
  self.frame.texture = texture;

  self.frame:SetPoint("CENTER",0,0);
  self.frame:Hide();

  self.animationGroup = self.frame:CreateAnimationGroup()
  self.animationGroup:SetLooping("BOUNCE")

  local animation = self.animationGroup:CreateAnimation("Alpha")
  animation:SetDuration(0.75)
  animation:SetFromAlpha(0.1)
  animation:SetToAlpha(0.9)
end

function OrganizedPanic:PlayAlertSound()
  if self.db.profile.playSound then
    PlaySoundFile(NS.SOUNDS[self.db.profile.sound], "Master")
  end
end

function OrganizedPanic:ShowRaidWarning(caster, spell)
    local message = ("%s casting %s"):format(caster, spell)
    RaidNotice_AddMessage(RaidWarningFrame, message, ChatTypeInfo["RAID_WARNING"])
end

function OrganizedPanic:FlashBorder()
  if self.db.profile.flashBorder then
    self:StartFlashing()
  end
end

function OrganizedPanic:StartFlashing()
  if self.animationGroup:IsPlaying() then self:StopFlashing() end
  self.frame.texture:SetVertexColor(self.db.profile.borderColor.r, self.db.profile.borderColor.g, self.db.profile.borderColor.b);
  self.animationGroup:Play()
  self.frame:Show()
  if self.flashTicker then self.flashTicker:Cancel() end
  self.flashTicker = C_Timer.NewTicker(4, function() self:StopFlashing() end, 0)
end

function OrganizedPanic:StopFlashing()
  if self.flashTicker then self.flashTicker:Cancel() end
  self.flashTicker = nil
  self.frame:Hide()
  self.animationGroup:Stop()
end

function OrganizedPanic:GetOption(info)
  -- print(ObjectToString(info.option))
  local property = info.option.arg
  return self.db.profile[property]
end

function OrganizedPanic:SetOption(info, value)
  -- print(ObjectToString(info.option), value)
  local property = info.option.arg
  self.db.profile[property] = value
end

function OrganizedPanic:SetBorderColor(info, r, g, b, a)
  self.db.profile.borderColor.r = r
  self.db.profile.borderColor.g = g
  self.db.profile.borderColor.b = b
  self:StartFlashing()
end

function OrganizedPanic:GetBorderColor(info)
  return self.db.profile.borderColor.r, self.db.profile.borderColor.g, self.db.profile.borderColor.b, 1.0
end

function OrganizedPanic:NSPrint(format, ...)
  print(PrefixString(NS.ADDON_NAME, string.format(format, ...), NS.PREFIX_COLOR))
end

function OrganizedPanic:HandleChatCommand(input)
  if not input or input:trim() == "" then
    LibStub("AceConfigDialog-3.0"):Open("OrganizedPanic")
  else
    if input == "help" then
      self:NSPrint("%s", "Commands:")
    else
      self:NSPrint("unknown command: '%s', try '/%s help'", input, NS.CHAT_COMMAND_NAME)
    end
  end
end
